@import "../../less/bootstrap/variables.less";

#job_runner {
    h4 {
        margin-bottom: 30px;
        text-align: center;
    }

    .width15pc {
        width: 15%;
    }

    .width25pc {
        width: 25%;
    }

    .width75pc {
        width: 75%;
    }

    h5 {
        margin: 0px;
    }

    .suspended {
        text-decoration: line-through;
    }
    pre {
        font-size: 10px;
    }

    .run-performance-graph {
        height: 120px;
    }

    .filters {
        margin-bottom: 30px;
    }

    .margin-top {
        margin-top: 35px;
    }

    table {
        ul {
            margin-bottom: 0px;
        }
    }

}

.modal {
    width: 900px;
    margin-left: -450px;
}

// copy from .navbar .nav > li > a
// used to align non links properly in the navbar
.navbar .nav > li > span {
    display: block;
    float: none;
    margin: ((@navbarHeight - @baseLineHeight) / 2) + 1 15px ((@navbarHeight - @baseLineHeight) / 2);
    text-decoration: none;
}
