<section>
    <div class="row">
        <div class="hero-unit margin-top">
            <h1>No projects available :(</h1>
            <p>
                There are no projects available for you.
                This can be because there aren't any projects in Job-Runner, or you don't have permissions to one of the existing projects.
            </p>
        </div>
    </div>
</section>